@@ -15,11 +15,13 @@
 
 #include <stdlib.h>
 #include <stdio.h>
+#include <errno.h>
 #include <tcl.h>
 #include <pi-source.h>
 #include <pi-socket.h>
 #include <pi-dlp.h>
 #include <pi-memo.h>
+#include <pi-buffer.h>
 
 #if TCL_MAJOR_VERSION >= 8
 # define Objects
@@ -30,34 +32,37 @@ struct tcl_e {char *name; long value;};
 char buf[0xffff];
 
 static struct tcl_e domains[] = {
-	{"PI_AF_SLP", PI_AF_SLP},
-	{"AF_SLP", PI_AF_SLP},
-	{"SLP", PI_AF_SLP},
+	{"PI_AF_PILOT", PI_AF_PILOT},
+	{"AF_PILOT", PI_AF_PILOT},
+	{"PILOT", PI_AF_PILOT},
 	{0,0}};
 static struct tcl_e protocols[] = {
+	{"PI_PF_DEV", PI_PF_DEV},
 	{"PI_PF_SLP", PI_PF_SLP}, 
+	{"PI_PF_SYS", PI_PF_SYS}, 
 	{"PI_PF_PADP", PI_PF_PADP},
-	{"PI_PF_LOOP", PI_PF_LOOP},
+	{"PI_PF_NET", PI_PF_NET},
+	{"PI_PF_DLP", PI_PF_DLP},
+	{"PF_DEV", PI_PF_DEV},
 	{"PF_SLP", PI_PF_SLP}, 
+	{"PF_SYS", PI_PF_SYS}, 
 	{"PF_PADP", PI_PF_PADP},
-	{"PF_LOOP", PI_PF_LOOP},
+	{"PF_NET", PI_PF_NET},
+	{"PF_DLP", PI_PF_DLP},
+	{"DEV", PI_PF_DEV},
 	{"SLP", PI_PF_SLP}, 
+	{"SYS", PI_PF_SYS}, 
 	{"PADP", PI_PF_PADP},
-	{"LOOP", PI_PF_LOOP},
+	{"NET", PI_PF_NET},
+	{"DLP", PI_PF_DLP},
 	{0,0}};
 static struct tcl_e types[] = {
 	{"PI_SOCK_STREAM", PI_SOCK_STREAM},
-	{"PI_SOCK_DGRAM", PI_SOCK_DGRAM},
 	{"PI_SOCK_RAW", PI_SOCK_RAW},
-	{"PI_SOCK_SEQPACKET", PI_SOCK_SEQPACKET},
 	{"SOCK_STREAM", PI_SOCK_STREAM},
-	{"SOCK_DGRAM", PI_SOCK_DGRAM},
 	{"SOCK_RAW", PI_SOCK_RAW},
-	{"SOCK_SEQPACKET", PI_SOCK_SEQPACKET},
 	{"STREAM", PI_SOCK_STREAM},
-	{"DGRAM", PI_SOCK_DGRAM},
 	{"RAW", PI_SOCK_RAW},
-	{"SEQPACKET", PI_SOCK_SEQPACKET},
 	{0,0}};
 
 static int
@@ -134,8 +139,12 @@ struct Packer {
 void MemoUnpackCmd(Tcl_Interp * interp, char * buf, int len)
 {
 	struct Memo m;
-	
-	len = unpack_Memo(&m, buf, len);
+	pi_buffer_t *record;
+	record = pi_buffer_new(len);
+	memcpy(record->data, buf, len);
+	record->used = len;
+	len = unpack_Memo(&m, record, len);
+	pi_buffer_free(record);
 
 	Tcl_AppendElement(interp, m.text);
 	free_Memo(&m);
@@ -145,10 +154,22 @@ void MemoUnpackCmd(Tcl_Interp * interp, char * buf, int len)
 int MemoPackCmd(Tcl_Interp * interp, char * rec, char * buf, int len)
 {
 	struct Memo m;
-	
+	pi_buffer_t *record;
+	int used;
+	record = pi_buffer_new(0);	
+
 	m.text = rec;
 	
-	return pack_Memo(&m, buf, len);
+	pack_Memo(&m, record, memo_v1);
+	if (record->used <= len) {
+	  memcpy(buf,record->data,record->used);
+	  used = record->used;
+	} else {
+	  /* It didn't fit in their fixed size buffer */
+	  used = 0;
+	}
+	pi_record_free(record);
+	return used;
 }
 
 struct { char * name; packcmd pack, packai, packsi; unpackcmd unpack, unpackai, unpacksi; }
@@ -658,7 +679,7 @@ CreateSocket(Tcl_Interp * interp, int protocol, char * remote, int server)
 	int result;
 	int sock;
 	int type;
-	int domain = PI_AF_SLP;
+	int domain = PI_AF_PILOT;
 	struct pi_sockaddr * addr = 0;
 	int alen = 0;
 	
@@ -677,13 +698,13 @@ CreateSocket(Tcl_Interp * interp, int protocol, char * remote, int server)
 
 	statePtr->fd = sock;
 
-	if (domain == PI_AF_SLP) {
+	if (domain == PI_AF_PILOT) {
 		char * device;
 		device = remote;
 		alen = strlen(device)+1+4;
 		addr = (struct pi_sockaddr*)malloc(alen);
 		strcpy(addr->pi_device, device);
-		addr->pi_family = PI_AF_SLP;
+		addr->pi_family = PI_AF_PILOT;
 	}
 	printf("addr = %ld\n", (long)addr);
 
@@ -955,7 +976,7 @@ bindCmd(ClientData clientData, Tcl_Interp * interp, int argc, char * argv[])
 	Tcl_GetInt(interp, argv[1], &s);
 	family = tcl_enum(interp, argv[2], domains);
 	
-	if (family == PI_AF_SLP) {
+	if (family == PI_AF_PILOT) {
 		device = argv[3];
 		alen = strlen(device)+1+4;
 		addr = (struct pi_sockaddr*)malloc(alen);
@@ -1086,7 +1107,7 @@ GetRecordCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
 	int index;
 	int sock;
 	int len,attr,cat;
-	recordid_t id;
+	recordid_t id_;
 	
 	if (argc != 4 ) {
 		Tcl_SetResult(interp, "Usage: GetRecord socket dbhandle index", TCL_STATIC);
@@ -1104,7 +1125,7 @@ GetRecordCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
 		return TCL_ERROR;
 	}
 	
-	result = dlp_ReadRecordByIndex(tcl_socket(interp, argv[1]), handle, index, buf, &id, &len, &attr, &cat);
+	result = dlp_ReadRecordByIndex(tcl_socket(interp, argv[1]), handle, index, buf, &id_, &len, &attr, &cat);
 	
 	if (result<0) {
 		Tcl_SetResult(interp, dlp_strerror(result), TCL_STATIC);
@@ -1113,7 +1134,7 @@ GetRecordCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
 	
 	pack[sock].unpack(interp, buf, len);
 	Tcl_AppendInt(interp, index);
-	Tcl_AppendInt(interp, id);
+	Tcl_AppendInt(interp, id_);
 	Tcl_AppendInt(interp, attr);
 	Tcl_AppendInt(interp, cat);
 
@@ -1128,7 +1149,7 @@ SetRecordCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
 	int index;
 	int sock;
 	int len,attr,cat;
-	recordid_t id;
+	recordid_t id_;
 	
 	if (argc != 7 ) {
 		Tcl_SetResult(interp, "Usage: SetRecord socket dbhandle record id attr cat", TCL_STATIC);
@@ -1145,13 +1166,13 @@ SetRecordCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
 		return TCL_ERROR;
 	}
 	
-	Tcl_GetInt(interp, argv[4], &index); id = index;
+	Tcl_GetInt(interp, argv[4], &index); id_ = index;
 	Tcl_GetInt(interp, argv[5], &attr);
 	Tcl_GetInt(interp, argv[6], &cat);
 
 	len = pack[sock].pack(interp, argv[3], buf, 0xffff);
 	
-	result = dlp_WriteRecord(sock, handle, attr, id, cat, buf, len, &id);
+	result = dlp_WriteRecord(sock, handle, attr, id_, cat, buf, len, &id_);
 	
 	free(buf);
 	
@@ -1160,7 +1181,7 @@ SetRecordCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
 		return TCL_ERROR;
 	}
 	
-	Tcl_AppendInt(interp, (int)id);
+	Tcl_AppendInt(interp, (int)id_);
 
 	return TCL_OK;
 }
